<?php
/*
* 2007-2018 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
*  @author ST themes <www.sunnytoo.com>
*  @copyright 2018 ST themes team.
*/
class ImageManager extends ImageManagerCore
{
    const PREFIX_ST = 'ST_IMGICK_';
    
    public static function thumbnail($image, $cacheImage, $size, $imageType = 'jpg', $disableCache = true, $regenerate = false)
    {
        if (!self::isModuleEnabled() || !self::isImagickEnabled()) {
            return parent::thumbnail($image, $cacheImage, $size, $imageType, $disableCache, $regenerate);
        }
        self::statCacheClear($image);
        if (!file_exists($image)) {
            return '';
        }
        if ($regenerate && file_exists(_PS_TMP_IMG_DIR_.$cacheImage)) {
            @unlink(_PS_TMP_IMG_DIR_.$cacheImage);
        }
        if ($regenerate || !file_exists(_PS_TMP_IMG_DIR_.$cacheImage)) {
            if (!parent::checkImageMemoryLimit($image)) {
                return '';
            }
            $srcImage = new Imagick($image);
            $x = $srcImage->getImageWidth();
            $y = $srcImage->getImageHeight();
            $maxX = $size * 3;
            $ratioX = $x / ($y / $size);
            if ($ratioX > $maxX) {
                $ratioX = $maxX;
                $size = $y / ($x / $maxX);
            }
            self::resize2($image, _PS_TMP_IMG_DIR_.$cacheImage, $srcImage, $ratioX, $size, $imageType);
        }
        if (Context::getContext()->controller->controller_type == 'admin') {
            return '<img src="../img/tmp/'.$cacheImage.($disableCache ? '?time='.time() : '').'" alt="" class="imgm img-thumbnail" />';
        } else {
            return '<img src="'._PS_TMP_IMG_.$cacheImage.($disableCache ? '?time='.time() : '').'" alt="" class="imgm img-thumbnail" />';
        }
    }
    
    public static function resize($srcFile, $dstFile, $dstWidth = null, $dstHeight = null, $fileType = 'jpg',
        $forceType = false, &$error = 0, &$tgtWidth = null, &$tgtHeight = null, $quality = 5,
        &$srcWidth = null, &$srcHeight = null)
    {
        if (!self::isModuleEnabled()) {
            return parent::resize($srcFile, $dstFile, $dstWidth, $dstHeight, $fileType,
                $forceType, $error, $tgtWidth , $tgtHeight, $quality, $srcWidth, $srcHeight);
        }
        
        if (!self::isImagickEnabled()) {
            return parent::resize($srcFile, $dstFile, $dstWidth, $dstHeight, $fileType,
                $forceType, $error, $tgtWidth , $tgtHeight, $quality, $srcWidth, $srcHeight);
        }
        self::statCacheClear($srcFile);
        if (!file_exists($srcFile) || !filesize($srcFile)) {
            return !($error = self::ERROR_FILE_NOT_EXIST);
        }
        if (!parent::checkImageMemoryLimit($srcFile)) {
            return !($error = self::ERROR_MEMORY_LIMIT);
        }
        // Copy source images to reduce original quantity loss.
        if (preg_match('/\/img\/(p|c|m|s|st)\/(\d+\/)*\d+\.(jpg|jpeg|png|gif)$/Us', $dstFile)) {
            return @copy($srcFile, $dstFile);
        }

        if (Configuration::get(self::PREFIX_ST.'CROP') && !self::getFileImageType($dstFile)) {
            return self::cut(
                $srcFile, 
                $dstFile, 
                $dstWidth, 
                $dstHeight, 
                $fileType, 
                0, 
                0
            );
        }

        $srcImage = new Imagick($srcFile);
        $srcWidth = $srcImage->getImageWidth();
        $srcHeight = $srcImage->getImageHeight();
        $trimWidth = $srcImage->getImageWidth();
        $trimHeight = $srcImage->getImageHeight();
        if (!$dstWidth) {
            $dstWidth = $trimWidth;
        }
        if (!$dstHeight) {
            $dstHeight = $trimHeight;
        }
        $ps_image_generation_method = Configuration::get('PS_IMAGE_GENERATION_METHOD');
        if ($ps_image_generation_method && ($trimWidth > $dstWidth || $trimHeight > $dstHeight)) {
            if ($ps_image_generation_method == 2) {
                $dstWidth = (int)(round(($trimWidth * $dstHeight) / $trimHeight));
            } else {
                $dstHeight = (int)(round(($trimHeight * $dstWidth) / $trimWidth));
            }
        }
        return self::resize2($srcFile, $dstFile, $srcImage, $dstWidth, $dstHeight, $fileType, $forceType);
    }
    
    private static function resize2($srcFile, $dstFile, $srcImage, $dstWidth, $dstHeight, $fileType = 'jpg', $force_type = false) {
        $srbgProfile = null;
        $srbgProfileBase64 = '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';
        
        $type = $srcImage->getImageFormat();
        if (Configuration::get('PS_IMAGE_QUALITY') == 'png_all'
            || (Configuration::get('PS_IMAGE_QUALITY') == 'png' && $type == 'PNG') && !$force_type) {
            $fileType = 'png';
        }
        $resize_gif = Configuration::get(self::PREFIX_ST.'RESIZE_GIF');
        if ($fileType == 'png') {
            $srcImage->setImageCompression(Imagick::COMPRESSION_LZW);
            $srcImage->setImageFormat('png');
            $png_quality = (int)Configuration::get(self::PREFIX_ST.'PNG_QUALITY');
            $srcImage->setImageCompressionQuality(($png_quality?:7) * 10 + (int)Configuration::get(self::PREFIX_ST.'IMAGICK_PNG_DATA_ENCODING'));
            $destTypeFile = 'png:'.$dstFile;
        } elseif (strtolower($type) == 'gif' && $resize_gif) {
            $destTypeFile = 'gif:'.$dstFile;
            $fileType = 'gif';
        } else {
            $srcImage->setImageCompression(Imagick::COMPRESSION_JPEG);
            $srcImage->setImageFormat('jpeg');
            $jpeg_quality = (int)Configuration::get(self::PREFIX_ST.'JPEG_QUALITY');
            // $srcImage->setSamplingFactors(array('2x2', '1x1', '1x1'));
            $srcImage->setImageCompressionQuality(($jpeg_quality?:90));
            if (Configuration::get(self::PREFIX_ST.'IMAGICK_PROGRESSIVE_JPEG')) {
                $srcImage->setInterlaceScheme(Imagick::INTERLACE_PLANE);
            } else {
                $srcImage->setInterlaceScheme(Imagick::INTERLACE_NO);
            }
            $destTypeFile = 'jpeg:'.$dstFile;
        }
        if ($fileType == 'png' && $type == 'PNG') {
            $srcImage->setImageBackgroundColor('none');
        } elseif ($fileType == 'gif' && $resize_gif) {

        } else {
            $srcImage->setImageBackgroundColor(Configuration::get(self::PREFIX_ST.'BACKGROUND_COLOR', null, null, null, 'white'));
        }
        Hook::exec(
            'actionChangeImagickSettings',
            [
                'imagick'    => &$srcImage,
                'src_file'   => $srcFile,
                'dst_file'   => $dstFile,
                'dst_width'  => $dstWidth,
                'dst_height' => $dstHeight,
                'file_type'  => $fileType,
            ]
        );
        if (Configuration::get(self::PREFIX_ST.'IMAGICK_STRIP_ICC_PROFILE') && !$resize_gif) {
            $iccModel = $srcImage->getImageProperty('icc:model');
            $colorSpace = $srcImage->getImageColorspace();
            if ($iccModel) {
                if (strpos($iccModel, 'sRGB') === false
                    && strpos($iccModel, 'c2') === false
                    && !($colorSpace == Imagick::COLORSPACE_SRGB && strpos($iccModel, 'SWOP') !== false)) {
                    if ($srbgProfile == null) {
                        $srbgProfile = base64_decode($srbgProfileBase64);
                    }
                    $srcImage->profileImage('icc', $srbgProfile);
                }
            } else {
                if ($colorSpace != Imagick::COLORSPACE_GRAY && $colorSpace != Imagick::COLORSPACE_SRGB) {
                    $srcImage->transformImageColorspace(Imagick::COLORSPACE_SRGB);
                }
            }
            $srcImage->stripImage();
        }
        $x = $srcImage->getImageWidth();
        $y = $srcImage->getImageHeight();
        if ($x != $dstWidth || $y != $dstHeight) {
            $srcImage = self::resizeImage($srcImage, $dstWidth, $dstHeight, $fileType);
        }
        if ($fileType == 'gif' && $resize_gif) {
            $writeFile = $srcImage->writeImages($destTypeFile, true);
        } else {
            $writeFile = $srcImage->writeImage($destTypeFile);
        }
        
        $srcImage->clear();
        $srcImage->destroy();
        Hook::exec('actionOnImageResizeAfter', ['dst_file' => $dstFile, 'file_type' => $fileType]);
        return $writeFile;
    }

    public static function cut($srcFile, $dstFile, $dstWidth = null, $dstHeight = null, $fileType = 'jpg', $dstX = 0, $dstY = 0)
    {
        if (!file_exists($srcFile)) {
            return false;
        }

        list($srcWidth, $srcHeight, $type) = getimagesize($srcFile);
        if (!$dstWidth) {
            $dstWidth = $srcWidth;
        }
        if (!$dstHeight) {
            $dstHeight = $srcHeight;
        }

        if ($type == IMAGETYPE_PNG) {
            $quality = (int)Configuration::get(self::PREFIX_ST.'PNG_QUALITY', null, null, null, 7);
            if ($quality < 0 || $quality > 10) {
                $quality = 7;
            }
            $quality = ($quality?:7) * 10 + (int)Configuration::get(self::PREFIX_ST.'IMAGICK_PNG_DATA_ENCODING');
        } else {
            $quality = (int)Configuration::get(self::PREFIX_ST.'JPEG_QUALITY', null, null, null, 90);
        }
        
        require_once(_PS_MODULE_DIR_.'stimagick/classes/PHPThumb/GD.php');
        $options = array('jpegQuality' => $quality);
        $thumb = new PHPThumb\GD($srcFile, $options);
        if ($srcWidth < $dstWidth || $srcHeight < $dstHeight) {
            $color = Configuration::get(self::PREFIX_ST.'BACKGROUND_COLOR', null, null, null, '#ffffff');
            $thumb->pad($dstWidth, $dstHeight, self::hex2rgb($color)); 
        }
        $thumb->adaptiveResize($dstWidth, $dstHeight);
        $thumb->save($dstFile);
        return ImageManager::isRealImage($dstFile);
    }

    public static function resizeImage($srcImage, $dstWidth, $dstHeight, $fileType='jpg')
    {
        if (!is_object($srcImage) || !$dstWidth || !$dstHeight) {
            return false;
        }

        $srcWidth = $srcImage->getImageWidth();
        $srcHeight= $srcImage->getImageHeight();
        list($newWidth, $newHeight) = self::CalcDimensions($srcWidth, $srcHeight, $dstWidth, $dstHeight);
        $dstImage = new Imagick();

        if ($fileType == 'gif' && Configuration::get(self::PREFIX_ST.'RESIZE_GIF')) {
            $srcImage = $srcImage->coalesceImages();
            foreach($srcImage as $frame)
            {
                $img = new Imagick();
                $img->newImage($dstWidth, $dstHeight, Configuration::get(self::PREFIX_ST.'BACKGROUND_COLOR', null, null, null, 'white'), 'gif');
                $frame->resizeImage($newWidth, $newHeight, Configuration::get(self::PREFIX_ST.'IMAGICK_FILTER', null, null, null, Imagick::FILTER_LANCZOS), (float)Configuration::get(self::PREFIX_ST.'IMAGICK_BLUR', null, null, null, 1), true);
                $img->compositeImage($frame, imagick::COMPOSITE_OVER, ($dstWidth - $newWidth) / 2, ($dstHeight - $newHeight) / 2);

                $dstImage->addImage($img);
                $dstImage->setImageDelay($img->getImageDelay());
                $dstImage->setImagePage($dstWidth, $dstHeight, 0, 0);

                $img->destroy();
            }
            $srcImage->destroy();
        } else {
            $srcImage->resizeImage($newWidth, $newHeight, Configuration::get(self::PREFIX_ST.'IMAGICK_FILTER', null, null, null, Imagick::FILTER_LANCZOS), (float)Configuration::get(self::PREFIX_ST.'IMAGICK_BLUR', null, null, null, 1), true);
            $dstImage->newImage($dstWidth, $dstHeight, $fileType == 'png' ? 'none' : Configuration::get(self::PREFIX_ST.'BACKGROUND_COLOR', null, null, null, 'white'));
            $dstImage->setImageFormat($fileType);
            $dstImage->compositeImage($srcImage, imagick::COMPOSITE_OVER, ($dstWidth - $newWidth) / 2, ($dstHeight - $newHeight) / 2);
            $srcImage->destroy();    
        }
        
        return $dstImage;
    }

    public static function CalcDimensions( $srcWidth, $srcHeight, $dstWidth = 0, $dstHeight = 0 )
    {
        if ( !$dstWidth && !$dstHeight )
            return array( $srcWidth, $srcHeight );

        $width_ratio = $height_ratio = 1.0;
        $did_width = $did_height = false;

        if ( $dstWidth > 0 && $srcWidth > 0 && $srcWidth > $dstWidth ) {
            $width_ratio = $dstWidth / $srcWidth;
            $did_width = true;
        }

        if ( $dstHeight > 0 && $srcHeight > 0 && $srcHeight > $dstHeight ) {
            $height_ratio = $dstHeight / $srcHeight;
            $did_height = true;
        }

        // Calculate the larger/smaller ratios
        $smaller_ratio = min( $width_ratio, $height_ratio );
        $larger_ratio  = max( $width_ratio, $height_ratio );

        if ( (int) round( $srcWidth * $larger_ratio ) > $dstWidth || (int) round( $srcHeight * $larger_ratio ) > $dstHeight ) {
            // The larger ratio is too big. It would result in an overflow.
            $ratio = $smaller_ratio;
        } else {
            // The larger ratio fits, and is likely to be a more "snug" fit.
            $ratio = $larger_ratio;
        }

        // Very small dimensions may result in 0, 1 should be the minimum.
        $w = max ( 1, (int) round( $srcWidth  * $ratio ) );
        $h = max ( 1, (int) round( $srcHeight * $ratio ) );

        // Sometimes, due to rounding, we'll end up with a result like this: 465x700 in a 177x177 box is 117x176... a pixel short
        // We also have issues with recursive calls resulting in an ever-changing result. Constraining to the result of a constraint should yield the original result.
        // Thus we look for dimensions that are one pixel shy of the max value and bump them up

        // Note: $did_width means it is possible $smaller_ratio == $width_ratio.
        if ( $did_width && $w == $dstWidth - 1 ) {
            $w = $dstWidth; // Round it up
        }

        // Note: $did_height means it is possible $smaller_ratio == $height_ratio.
        if ( $did_height && $h == $dstHeight - 1 ) {
            $h = $dstHeight; // Round it up
        }

        return array( $w, $h );
    }
    public static function hex2rgb($hex) {
       $hex = str_replace("#", "", $hex);
    
       if(strlen($hex) == 3) {
          $r = hexdec(substr($hex,0,1).substr($hex,0,1));
          $g = hexdec(substr($hex,1,1).substr($hex,1,1));
          $b = hexdec(substr($hex,2,1).substr($hex,2,1));
       } else {
          $r = hexdec(substr($hex,0,2));
          $g = hexdec(substr($hex,2,2));
          $b = hexdec(substr($hex,4,2));
       }
       $rgb = array($r, $g, $b);
       return $rgb;
    }

    public static function getFileImageType($file)
    {
        $imageType = false;
        if (!$file) {
            return $imageType;
        }
        $iso = Context::getContext()->language->iso_code;
        if (preg_match('/\/(\d+|'.$iso.'\-default)\-(\w+\_\w+(\_\w+)*)(\/|\.)/Us', $file, $match)) {
            $imageType = isset($match[2]) ? $match[2] : '';
        }
        return $imageType;
    }
    
    private static function isModuleEnabled() {
        if (!Module::isEnabled('stimagick')) {
            return false;
        }
        if (!class_exists('stimagick')) {
            require_once _PS_MODULE_DIR_.'stimagick/stimagick.php';
        }
        return true;
    }
    
    private static function isImagickEnabled() {
        return extension_loaded('imagick');
    }
    
    private static function statCacheClear($image) {
        if (PHP_VERSION_ID < 50300) {
            clearstatcache();
        } else {
            clearstatcache(true, $image);
        }
    }
}
